/*
** Der umstlpbare Wrfel nach Paul Schatz
** (C) 1999 Franz Zahaurek <http://www.fzk.at>
**
** Dieses Programm samt Dokumentation ist Freie Software und kann beliebig
** verwendet werden, soferne der obige Copyright-Vermerk nicht entfernt
** wird.
*/
/* $id$ */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef OS_WIN32
#include <sys/time.h>
#include <unistd.h>
#endif
#include <sys/types.h>
#include <math.h>
#define DEFINECONSTANTS
#include "wuerfel.h"

int main(int argc, char **argv) {

    ident ();
    usage ();

    glutInit (&argc, argv);

    setoptiondefaults ();

    getoptions (argc, argv);

    berechnewg();

    if (options.print) {
        printf ("Datenfiles enthalten %d Positionen des Wrfelgrtels\n",
                anzelt);
    }
    else {
        control ();
        owgp = wg;
        zeichnewg();
    }

    freewg();

    return 0;
}

void
ident (void) {
    fprintf (stderr, "Der umstuelpbare Wuerfel nach Paul Schatz, Version: %s\n", VERSION);
    fprintf (stderr, "(C) 2000 Franz Zahaurek <http://www.fzk.at>\n");
}

void
usage (void) {
    fprintf (stderr, "\n");
    fprintf (stderr, "Optionen: -a     Achsen darstellen\n");
    fprintf (stderr, "          -b #   Breite des Fensters in Pixeln, Default=800\n");
    fprintf (stderr, "          -e     Linien zum Evolutions- und Involutionspunkt darstellen\n");
    fprintf (stderr, "          -f     Flaechen darstellen (Default)\n");
    fprintf (stderr, "          -g     Gittermodell darstellen\n");
    fprintf (stderr, "          -h     Hilfetext ausgeben\n");
    fprintf (stderr, "          -i     Info aus\n");
    fprintf (stderr, "          -l     Labels darstellen\n");
    fprintf (stderr, "          -m #   Animationsmodus, Default=1\n");
    fprintf (stderr, "                     1...Hauptdiagonale fixiert\n");
    fprintf (stderr, "                     2...Scharnierkante fixiert\n");
    fprintf (stderr, "                     3...Element fixiert\n");
    fprintf (stderr, "          -p     nur Koordinaten der Punkte ausgeben\n");
    fprintf (stderr, "          -t     Wuerfelguertel schwebt, Default=nicht schweben\n");
    fprintf (stderr, "          -w #   Schrittweite in Grad, Default=6 Grad\n");
}

void
control (void) {
    fprintf (stderr, "\n");
    fprintf (stderr, "Steuerung mit der Maus:\n\n");
    fprintf (stderr, "  Rechte_Maustaste        Kontextmenue\n\n");
    fprintf (stderr, "Steuerung mit Hot-Keys:\n\n");
    fprintf (stderr, "  i            Toggle: Info ein/aus\n");
    fprintf (stderr, "  t            Toggle: Schweben ein/aus\n");
    fprintf (stderr, "  -/+          schneller/langsamer drehen\n");
    fprintf (stderr, "  g            Toggle: Gitter-/Flaechendarstellung\n");
    fprintf (stderr, "  l            Toggle: Labels ein/aus\n");
    fprintf (stderr, "  s            Toggle: start/stopp\n");
    fprintf (stderr, "  ./,          Naechste/vorherige Position\n");
    fprintf (stderr, "  x/y/z        Drehung um die x/y/z-Achse\n");
    fprintf (stderr, "  1/2/3        Drehmodus: Diagonale/Scharnier/Element fxiert\n");
    fprintf (stderr, "  <HOME>       Drehung um x/y/z-Achse auf 0 stellen\n");
    fprintf (stderr, "  e            Toggle: Linien zu E/I zeigen/verbergen\n");
    fprintf (stderr, "  a            Toggle: Achsen zeigen/verbergen\n");
    fprintf (stderr, "  o            Toggle: Oloid zeigen/verbergen\n");
    fprintf (stderr, "  Pfeiltasten  Verschieben in x/y Richtung\n");
    fprintf (stderr, "  <PAGE_UP>,<PAGE_DOWN>  Verschieben in z Richtung\n");
    fprintf (stderr, "  <ESC>,q      Quit\n\n");
}

#ifdef OS_WIN32
void
warten (int sec) {
	return;
}


#else
void
warten (int sec){
    fd_set rfds;
    struct timeval tv;

    /* Watch stdin (fd 0) to see when it has input. */
    FD_ZERO(&rfds);
    FD_SET(0, &rfds);

    /* Wait up to sec seconds. */
    tv.tv_sec = sec;
    tv.tv_usec = 0;

    select(1, &rfds, NULL, NULL, &tv);
}
#endif

#ifdef OS_WIN32
/*
** Der Algorithmus zur Abarbeitung von Optionen entstammt dem Buch:
** A.T.Schreiner: System-Programmierung in UNIX, Teil 1: Werkzeuge
** Seite 136 ff
** Verlag: B.G. Teubner Stuttgart 1984
**
** Anmerkung: Man beachte die Konstruktionen: **++argv und *++*argv
*/
#define LOW  'a'
#define HIGH 'z'
#define VAL(x) vals[(x) - LOW]
static int vals[HIGH - LOW + 1];
void
getoptions (int argc, char **argv) {

    while (--argc) {
	char i;

	if (**++argv == '-') {

	    switch (i = *++*argv) {
	    case 0:
		/* alleinstehendes Minus wird ebergangen */
		fprintf (stderr, "alleinstehendes Minus\n");
		continue ;
	    case 'b':
	    case 'm':
	    case 'w':
		if (*++*argv) {
		    VAL (i) = atoi (*argv);
		    fprintf (stderr, "Wert %c = %d\n", i, VAL(i));
		}
		else if (-- argc) {
		    VAL (i) = atoi (*++argv);
		    fprintf (stderr, "Wert %c = %d\n", i, VAL(i));
		}
		else {
		    /* Wert zur Option fehlt */
		    fprintf (stderr, "Wert zur Option %c fehlt\n", i);
		    exit (1);
		    break;
		}
		switch (i) {
		case 'b':
		    options.seite = VAL (i);
		    break;
		case 'm':
		    options.mode = VAL (i);
		    if (options.mode < 1 || options.mode > 3) {
			options.mode = 1;
		    }
		    break;
		case 'w':
		    options.dw = TOBWINKEL((double) VAL (i));
		    break;
		}
		continue;

	    case 'a':
		options.ach = TRUE;
		continue;

	    case 'e':
		options.ei = TRUE;
		continue;

	    case 'f':
		options.lines = FALSE;
		polygonMode = GL_FILL;
		continue;

	    case 'g':
		options.lines = TRUE;
		polygonMode = GL_LINE;
		continue;

	    case 'h':
		/* usage (); */
		exit (1);

	    case 'i':
		options.info = FALSE;
		continue;

	    case 'p':
		options.print = TRUE;
		continue;

	    case 't':
		options.tork = TRUE;
		continue;

	    case 'l':
		options.labels = TRUE;
		continue;

	    default:
		/* Unbekannte Option */
		fprintf (stderr, "A Option %c\n", i);
		exit (1);
		continue;
	    }
	    break;
	}
        else {
		fprintf (stderr, "Unbekanntes Argument %s\n", *argv);
		exit (1);
	}
    }
}
#else
void
getoptions (int argc, char **argv) {
    int c;
    extern int optind;

    optind = 1;
    while (TRUE) {
        extern int opterr, optopt;
        opterr = 0;

        c = getopt (argc, argv, "ab:efghilm:ptw:");
/*      fprintf (stderr, "getopt: c=%d, opterr=%d, optopt=%c, optarg=%s\n", */
/*               c, opterr, optopt, optarg); */
        if ( c == -1) {
            break;
        }

        switch (c) {
        case -1:
            break;

        case 'a':
            options.ach = TRUE;
            break;

        case 'b':
            options.seite = atoi (optarg);
            break;

        case 'e':
            options.ei = TRUE;
            break;

        case 'f':
            options.lines = FALSE;
            break;

        case 'g':
            options.lines = TRUE;
	    polygonMode = GL_LINE;
            break;

        case 'h':
            /* usage (); */
            exit (1);

	case 'i':
	    options.info = FALSE;
	    break;

        case 'm':
            options.mode = atoi (optarg);
	    if (options.mode < 1 || options.mode > 3) {
		options.mode = 1;
	    }
            break;

        case 'p':
            options.print = TRUE;
            break;

        case 't':
            options.tork = TRUE;
            break;

        case 'w':
            options.dw = TOBWINKEL((double)atoi (optarg));
            break;

        case 'l':
            options.labels = TRUE;
            break;

        default:
            fprintf (stderr, "\n");
            fprintf (stderr, "Falsche Option oder fehlerhaftes Optionsargument: <%c>\n", optopt);
            /* usage (); */
            exit (1);
        }
    }
}
#endif

void
setoptiondefaults (void) {
    /* Defaults */
    options.dw = 0;
    options.seite = 0;
    options.print = FALSE;
    options.ei = FALSE;
    options.ach = FALSE;

    options.lines = FALSE;
    polygonMode = GL_FILL;

    options.mode = 1;
    options.info = TRUE;
    options.tork = FALSE;
    options.perspektive = TRUE;
    options.labels = FALSE;
}

void
freewg (void) {
    WG *wgp;

    wgp = wg;
    while (wgp) {
        WG *t;

        t = wgp;
        wgp = wgp->nextwgp;
        free (t);
    }
}
